/*
* file CcProject.java
*
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
* 
* com.ibm.rational.wvcm.stp.cc.CcProject
*
* (C) Copyright IBM Corporation 2004, 2011.  All Rights Reserved. 
* Note to U.S. Government Users Restricted Rights:  Use, duplication or  
* disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
*/

package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.Feedback;
import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;

import com.ibm.rational.wvcm.stp.cq.CqRecordType;
import com.ibm.rational.wvcm.stp.cq.CqUserDb;
import com.ibm.rational.wvcm.stpex.StpExEnumeration;

/**
 * <p>A proxy for a ClearCase UCM project.
 * </p>
 * <p>A UCM project contains the configuration information needed to manage a
 * significant development effort, such as a product release. A project contains
 * one main shared <i>integration stream</i> and typically multiple per-user
 * <i>development streams</i>.
 * </p>
 * <p>A UCM project may be <i>ClearQuest-enabled </i> by linking it to a particular
 * ClearQuest database. In ClearQuest-enabled projects, all UCM activities are
 * associated with ClearQuest entities in the ClearQuest database. ClearQuest
 * provides richer activity management functionality than is available from
 * ClearCase alone.
 * </p>
 * <p>
 * For more project information, see the ClearCase "Guide to Developing Software with UCM"
 * manual, and the cleartool man page "mkproject".
 * </p>
 */
public interface CcProject extends CcVobResource {

    /**
     * A UCM project is "ClearQuest-enabled" if it is bound to a ClearQuest
     * user database and participates in the ClearQuest/UCM integration.
     */
    enum ClearQuestEnabledState implements StpExEnumeration  {

        /**
         * This project is not ClearQuest-enabled.
         */
        NOT_ENABLED,

        /**
         * This project is ClearQuest-enabled.
         */
        ENABLED,

        /**
         * This project is ClearQuest-enabled, but the integration is
         * temporarily suspended.
         */
        SUSPENDED;
    }
    
    /** 
     * Defines the baseline name template for a project. 
     * The property specifies the tokens to be used in the baseline name and 
     * it can include any of the following tokens separated by commas:
     * <ul>
     * <li>basename
     * <li>project
     * <li>stream
     * <li>component
     * <li>date
     * <li>time
     * <li>user
     * <li>host
     * </ul>
     * When the baseline is created, UCM replaces commas with underscores in the baseline name.
     */
    PropertyName<String> BASELINE_NAMING_TEMPLATE =
        new PropertyName<String>(PROPERTY_NAMESPACE, "baseline-naming-template");
    
    /**
     * Get the value of this proxy's {@link #BASELINE_NAMING_TEMPLATE} property.
     * @return a string containing this projects baseline naming template
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    String getBaselineNamingTemplate() throws WvcmException;

    /**
     * Set the template to be used when naming a newly created baseline.
     * Specifying an empty string for a template will result in the default
     * template being set (currently "basename").
     * @param bl_template A string containing the desired naming template
     */
    void setBaselineNamingTemplate(String bl_template);
    
    /** This UCM project's integration stream. */
    PropertyName<CcStream> INTEGRATION_STREAM =
        new PropertyName<CcStream>(PROPERTY_NAMESPACE, "integration-stream");

    /**
     * Get the value of this proxy's {@link #INTEGRATION_STREAM} property.
     * @return a client proxy for this project's integration stream,
     *     or null if project doesn't have an integration stream
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    CcStream getIntegrationStream() throws WvcmException;

    /** The list of streams in this UCM project. */
    PropertyName<ResourceList<CcStream>> STREAM_LIST =
        new PropertyName<ResourceList<CcStream>>(PROPERTY_NAMESPACE, "stream-list");

    /**
     * Get the value of this proxy's {@link #STREAM_LIST} property.
     * @return list of project's streams as a ResourceList of UcmStream instances.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    ResourceList<CcStream> getStreamList() throws WvcmException;


    /** The list of UCM enabled CQ record types in this UCM project. */
    PropertyName<ResourceList<CqRecordType>> UCM_ENABLED_CQ_RECORD_TYPE_LIST =
        new PropertyName<ResourceList<CqRecordType>>(PROPERTY_NAMESPACE, "ucm-enabled-cq-record-type-list");

    /**
     * Get the value of this proxy's {@link #UCM_ENABLED_CQ_RECORD_TYPE_LIST} property.
     * @return list of project's ucm enabled cq record types as a ResourceList of CqRecordType instances.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    ResourceList<CqRecordType> getUcmEnabledCqRecordTypeList() throws WvcmException;

    /** The list of UCM components that can be modified in the UCM stream(s)
     * associated with this UCM project. */
    PropertyName<ResourceList<CcComponent>> MODIFIABLE_COMPONENT_LIST =
        new PropertyName<ResourceList<CcComponent>>(PROPERTY_NAMESPACE, "modifiable-component-list");

    /**
     * Get the value of this proxy's {@link #MODIFIABLE_COMPONENT_LIST} property.
     * @return list of modifiable components as ResourceList of UcmComponent instances.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    ResourceList<CcComponent> getModifiableComponentList() throws WvcmException;

    /** Is this a single-stream UCM project? */
    PropertyName<Boolean> IS_SINGLE_STREAM =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-single-stream");

    /**
     * Get the value of this proxy's {@link #IS_SINGLE_STREAM} property.
     * @return true if this is a single-stream UCM project, else false.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    boolean getIsSingleStream() throws WvcmException;

    /**
     * Set whether the project is single stream or not. 
     * This property can only be set at project creation time.
     * @param isSingleStream true if the project should be single stream, false otherwise.
     */
    void setIsSingleStream(boolean isSingleStream);

    /** Is this a ClearQuest-enabled UCM project? */
    PropertyName<ClearQuestEnabledState> CLEARQUEST_ENABLED_STATE =
        new PropertyName<ClearQuestEnabledState>(PROPERTY_NAMESPACE, "clearquest-enabled-state");

    /**
     * Get the value of this proxy's {@link #CLEARQUEST_ENABLED_STATE} property.
     * @return ENABLED, NOT_ENABLED, or SUSPENDED depending on this project's state.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    ClearQuestEnabledState getClearQuestEnabledState() throws WvcmException;

    /** The ClearQuest user database this CQ-enabled UCM project is associated with. */
    PropertyName<CqUserDb> CLEARQUEST_USER_DB =
        new PropertyName<CqUserDb>(PROPERTY_NAMESPACE, "clearquest-user-db");

    /**
     * Get the value of this proxy's {@link #CLEARQUEST_USER_DB} property.
     * @return a proxy for the CqUserDb associated with this CQ-enabled
     * UCM project, or null if project is not CQ-enabled
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    CqUserDb getClearQuestUserDb() throws WvcmException;

    /** The UCM folder in which this UCM project resides */
    PropertyName<CcProjectFolder> PROJECT_FOLDER =
        new PropertyName<CcProjectFolder>(PROPERTY_NAMESPACE, "project-folder");

    /**
     * Get the value of this proxy's {@link #PROJECT_FOLDER} property.
     * @return A client proxy for the UCM folder that contains this UCM project.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    CcProjectFolder getProjectFolder() throws WvcmException;

    /**
     * Set the parent folder for this project
     * @param folder client proxy for the parent project folder.
     */
    void setProjectFolder(CcProjectFolder folder);

    /** 
     * A list of all streams in this UCM project that have posted deliveries
     * in progress. A delivery is posted if the source stream is mastered
     * at a different replica than the target stream at the time of the delivery.
     */
    PropertyName<ResourceList<CcStream>> POSTED_DELIVERY_LIST =
        new PropertyName<ResourceList<CcStream>>(PROPERTY_NAMESPACE, "ccproject-posted-delivery-list");
    
    /**
     * Get the value of this proxy's {@link #POSTED_DELIVERY_LIST} property.
     * @return A list containing this project's posted deliveries.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    ResourceList<CcStream> getPostedDeliveryList() throws WvcmException;
    
    /**
     * Create a new UCM project at the location specified by this proxy. The
     * location should be an object name selector specifying the project's name
     * and the repository (project VOB) in which to create it.
     */
    CcProject doCreateCcProject(Feedback feedback) throws WvcmException;
}
